package no.uio.inf5750.assignment3.dashboard;

import java.nio.Buffer;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.LinkedList;
import java.util.TimeZone;

import no.uio.inf5750.assignment3.R;
import no.uio.inf5750.assignment3.accounts.Accounts;
import no.uio.inf5750.assignment3.interpretation.InterpretationActivityNew;
import no.uio.inf5750.assignment3.messaging.MessageListArrayAdapter;
import no.uio.inf5750.assignment3.messaging.MessagingActivity;
import no.uio.inf5750.assignment3.util.BaseActivity;
import no.uio.inf5750.assignment3.util.ConnectionManager;
import no.uio.inf5750.assignment3.util.Dashboard;
import no.uio.inf5750.assignment3.util.InterpretationInfoNode;
import no.uio.inf5750.assignment3.util.UpdateDaemon;
import no.uio.inf5750.assignment3.util.Interpretation;
import no.uio.inf5750.assignment3.util.Util;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.drawable.Drawable;

import android.os.Bundle;

import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup.LayoutParams;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.SlidingDrawer;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.Toast;

public class DashboardActivityNew extends BaseActivity {
	private Spinner mDashboardTitleSpinner;
	private ListView mListView; // contains all the full interpretations
	//private ListView mListViewSlidingDrawer;
	//private ListView mListViewSlidingDrawerAccounts;
	private Context mContext;
	//private Button mButtonDrawerHandle;
	//private Button mButtonAccounts;
	private Dashboard mDashboard;
	//private SlidingDrawer mSlidingDrawer;
	private int mCurrentDashboard = 0;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.dashboard);
		mContext = this;
		// Initializes variables
		setActivityObjects();

		if(!update())
			return;
		//Get the latest dashboard
		
		setSpinner();
	}
	
	/**
	 * Sets the items of the spinner
	 */
	public void setSpinner()
	{
		ArrayAdapter<String> spinner_adapter = new ArrayAdapter<String>(this, R.layout.spinner_header_big, UpdateDaemon.getDaemon().getDashboardsNames());//ArrayAdapter.createFromResource(this, R.array.game_speeds, R.layout.spinner_header);
    	spinner_adapter.setDropDownViewResource(R.layout.spinner_list_item);
    	mDashboardTitleSpinner.setAdapter(spinner_adapter);
    	mDashboardTitleSpinner.setEnabled(true);
	}

	/**
	 * Adds the dashboard items to a list
	 */
	public void setDashboardItems() {

		//Make a dummy string[] of the same length as the number of dashboard items
		if(mDashboard == null)
		{
			//Do nothing I guess... //TODO Make better feedback
			//Should set with an empty array adapter to remove previous dashboardItems
			String[] values = new String[]{"This dashboard is empty"};
			ArrayAdapter<String> adapter = new ArrayAdapter<String>(mContext, R.layout.list_item, values);
			mListView.setAdapter(adapter);
		}
		else
		{
			String[] values = new String[mDashboard.mDashboardItems.size()];
			DashboardListArrayAdapter adapter = new DashboardListArrayAdapter(mContext, R.layout.dashboard_list_item, values, mDashboard, this);
			mListView.setAdapter(adapter);
		}
		
	}

	// Activity Methods\\
	private boolean update() {
		// Contacts server to get current dashboards
		
		int verified = UpdateDaemon.getDaemon().verify();
		if(verified<11)
		{
			String error = "";
			if(verified==0) //00
			{
				error = "There is a problem with your account and your connection to the internet, please make sure your settings are correct";
				//connection error and account error
				System.out.println("double rror");
			}
			else if (verified==1) // 01
			{
				error = "There is a problem with your account. Please make sure your settings are correct";
				//account error but connection good
				System.out.println("account error");
			}
			else // 10
			{
				error = "There is a problem with your connection to the internet. Please make sure your connection settings are correct.";
				//connection error
				System.out.println("connection error");
			}
			
			showError(error);
			return false;
		}
		
		UpdateDaemon.getDaemon().updateDashboards();
		mDashboard = UpdateDaemon.getDaemon().getDashboard();
		
		// Shows the current interpretations
		// TODO: Limit to a number of interps at a time
		setDashboardItems();
		return true;
	}
	
	@Override
	public void showError(String error)
	{
		super.showError(error);
		setSpinnerError();
	}
	
	public void setSpinnerError()
	{
		String[] items = new String[]{"Error loading"};
		ArrayAdapter<String> spinner_adapter = new ArrayAdapter<String>(this, R.layout.spinner_header_big, items);
    	spinner_adapter.setDropDownViewResource(R.layout.spinner_list_item);
    	mDashboardTitleSpinner.setAdapter(spinner_adapter);
    	mDashboardTitleSpinner.setEnabled(false);
	}

	void setActivityObjects() {
		mListView = (ListView) findViewById(R.id.dashboard_listview);
		mDashboardTitleSpinner = (Spinner) findViewById(R.id.dashboard_spinner);
    	mDashboardTitleSpinner.setOnItemSelectedListener(new OnItemSelectedListener() 
    	{

			@Override
			public void onItemSelected(AdapterView<?> arg0, View arg1, int arg2, long arg3) 
			{
				int tempCurrent = mCurrentDashboard;
				mCurrentDashboard = arg2;
				UpdateDaemon.getDaemon().setCurrentDashboard(arg2);
				mDashboardTitleSpinner.setSelection(arg2);
				if(tempCurrent!=arg2)
				{//Have to check this or the update will loop
					update();
				}
			}
			@Override
			public void onNothingSelected(AdapterView<?> arg0) 
			{
				// TODO Auto-generated method stub
			}
		});
		
		Util.setSlidingDrawer(this);
	}
	
	public void switchAccount(int position)
	{
		System.out.println("switchAccounts");
		ConnectionManager.getConnectionManager().setAccount(position, mContext);
		UpdateDaemon.getDaemon().setCurrentDashboard(0);
		update();
		setSpinner();
	}
}
