package no.uio.inf5750.assignment3.interpretation;

import java.nio.Buffer;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.LinkedList;
import java.util.TimeZone;

import no.uio.inf5750.assignment3.R;
import no.uio.inf5750.assignment3.messaging.MessageListArrayAdapter;
import no.uio.inf5750.assignment3.util.BaseActivity;
import no.uio.inf5750.assignment3.util.ConnectionManager;
import no.uio.inf5750.assignment3.util.InterpretationInfoNode;
import no.uio.inf5750.assignment3.util.UpdateDaemon;
import no.uio.inf5750.assignment3.util.Interpretation;
import no.uio.inf5750.assignment3.util.Util;
import android.app.Activity;
import android.content.Context;
import android.graphics.drawable.Drawable;

import android.os.Bundle;

import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup.LayoutParams;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.Toast;

public class InterpretationActivityNew extends BaseActivity {
	private ListView mListView; //contains all the full interpretations
	private Context mContext;
	private Button mButtonRefresh;
	private LinkedList<Interpretation> mInterpretationList;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.interpretations_new);
		mContext = this;
		//Initializes variables
		setActivityObjects();
		
		//Updates interpretation list
		update();
	}
	
	/**
	 * adds interpretations to a list 
	 */
	public void setInterpretations()
	{
		//Create a dummy string array "values" of the same size as the number of interpretations, because we don't use the values array anyways.
		//Check if there are any interpretations first!
		if(mInterpretationList==null)
		{
			//There are no interpretations!
			String[] values = new String[]{"You have no interpretations"};
			ArrayAdapter<String> arrayAdapter = new ArrayAdapter<String>(mContext, R.layout.list_item, values);
			mListView.setAdapter(arrayAdapter);
			mListView.setClickable(false);
			mListView.setSelector(android.R.color.transparent);
			mListView.setOnItemClickListener(new OnItemClickListener() {
				public void onItemClick(AdapterView<?> parent, View view,
						int position, long id) {
						//Do nothing
					}
			});

		}
		else
		{
			String[] values = new String[mInterpretationList.size()];
			InterpretationListArrayAdapter arrayAdapter = new InterpretationListArrayAdapter(mContext, R.layout.interpretations_list_item, values, mInterpretationList);
			LinkedList<InterpretationInfoNode> tempList = mInterpretationList.get(0).mInfo;
			//System.out.println("#templist size: "+tempList.size());
			mListView.setAdapter(arrayAdapter);
		}
		
	}
	
	//Activity Methods\\
	private boolean update() {
		//Contacts server to get current interpretations
		int verified = UpdateDaemon.getDaemon().verify();
		if(verified<11)
		{
			String error = "";
			if(verified==0) //00
			{
				error = "There is a problem with your account and your connection to the internet, please make sure your settings are correct";
				//connection error and account error
				System.out.println("double rror");
			}
			else if (verified==1) // 01
			{
				error = "There is a problem with your account. Please make sure your settings are correct";
				//account error but connection good
				System.out.println("account error");
			}
			else // 10
			{
				error = "There is a problem with your connection to the internet. Please make sure your connection settings are correct.";
				//connection error
				System.out.println("connection error");
			}
			
			showError(error);
			return false;
		}
		
		
		UpdateDaemon.getDaemon().updateMessagesAndInterpretations();

		mInterpretationList = UpdateDaemon.getDaemon().getInterpretations();
    	//Shows the current interpretations 
		//TODO: Limit to a number of interps at a time - Actually is done automatically by dynamic loading in ListView
    	setInterpretations();
    	return true;
	}
	
	@Override
	public void showError(String error)
	{
		super.showError(error);
		
	}
	
	void setActivityObjects()
	{		
		mListView = (ListView) findViewById(R.id.interpretations_listview);

		Util.setSlidingDrawer(this);
	}	
	
	public void switchAccount(int position)
	{
		System.out.println("switchAccounts");
		ConnectionManager.getConnectionManager().setAccount(position, mContext);
		UpdateDaemon.getDaemon().setCurrentDashboard(0);
		update();
		
	}
}
