package no.uio.inf5750.assignment3.interpretation;

import java.util.LinkedList;

import no.uio.inf5750.assignment3.R;
import no.uio.inf5750.assignment3.util.Comment;
import no.uio.inf5750.assignment3.util.ConnectionManager;
import no.uio.inf5750.assignment3.util.Interpretation;
import no.uio.inf5750.assignment3.util.UpdateDaemon;
import android.content.Context;
import android.graphics.drawable.Drawable;
import android.os.AsyncTask;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;

public class InterpretationListArrayAdapter extends ArrayAdapter<String>{
	
	private Context mContext;
	private LinkedList<Interpretation> mInterpretationList;
	private LinearLayout mCommentsContainer;
	
	public InterpretationListArrayAdapter(Context context, int textViewResourceId,
			String[] objects, LinkedList<Interpretation> interpretations) {
		super(context, textViewResourceId, objects);
		this.mContext = context;
		this.mInterpretationList = interpretations;
	}
	
	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		LayoutInflater inflater = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		View rowView = inflater.inflate(R.layout.interpretations_list_item, parent, false);
		TextView textViewOriginalUsername = (TextView) rowView.findViewById(R.id.interpretations_list_original_username);
		TextView textViewOriginalDate = (TextView) rowView.findViewById(R.id.interpretations_list_original_date);
		//TextView textViewTitle = (TextView) rowView.findViewById(R.id.accounts_title);
		ProgressBar progressBar = (ProgressBar) rowView.findViewById(R.id.interpretations_list_item_progress);
		TextView textViewOriginalMessage = (TextView) rowView.findViewById(R.id.interpretations_list_original_message);
		ImageView imageView = (ImageView) rowView.findViewById(R.id.interpretations_list_image);
		mCommentsContainer = (LinearLayout) rowView.findViewById(R.id.interpretations_list_commentcontainer);
		final EditText editTextReply = (EditText) rowView.findViewById(R.id.interpretations_list_replyfield);
		Button buttonReply = (Button) rowView.findViewById(R.id.interpretations_list_replybutton);
		//Set username from original poster
		String originalPosterName = mInterpretationList.get(position).mUser.mName;
		textViewOriginalUsername.setText(originalPosterName);
		
		String originalPostDate = mInterpretationList.get(position).mCreated;
		textViewOriginalDate.setText(originalPostDate);
		
		//Set the original comment by OP
		String text = mInterpretationList.get(position).mText;
		textViewOriginalMessage.setText(text);
		
		//Set the image related to this interpretation conversation
		showChart(position, imageView, progressBar, rowView);
		
		//Set comments in linear layout
		showComments(mCommentsContainer, position, inflater, parent);
		final int position_test = position;
		buttonReply.setOnClickListener(new OnClickListener(){

			@Override
			public void onClick(View arg0) {
				// TODO Auto-generated method stub
				reply(position_test, editTextReply);
			}
			
		});
		return rowView;
	}
	
	public void reply(int position, EditText editText)
	{
		String message = editText.getText().toString();
		System.out.println("position: " + position);
		System.out.println("message:" + message);
		ConnectionManager.getConnectionManager().replyInterpretation(mInterpretationList.get(position).mId,
				 message);
		UpdateDaemon.getDaemon().updateMessagesAndInterpretations();
		//showComments(mCommentsContainer, position);
	}
	
	private void showComments(LinearLayout container, int position, LayoutInflater inflater, ViewGroup parent)
	{
		Interpretation interpretation = mInterpretationList.get(position);
		for(int i = 0; i < mInterpretationList.get(position).mCommentThread.size(); i++)
		{
			Comment comment = interpretation.mCommentThread.get(i);
			//User info
			LinearLayout commentsView = (LinearLayout) inflater.inflate(R.layout.interpretations_list_comment, parent, false);
			TextView username = (TextView) commentsView.findViewById(R.id.interpretations_list_comment_username);
			TextView date = (TextView) commentsView.findViewById(R.id.interpretations_list_comment_date);
			TextView message = (TextView) commentsView.findViewById(R.id.interpretations_list_comment_message);
			username.setText(comment.mUser.mName);
			date.setText(comment.mCreated);
			message.setText(comment.mText);

			container.addView(commentsView);
			
			Log.d("Log Comment Text:", mInterpretationList.get(position).mCommentThread.get(i).mText);
		}
	}
	
	private void showChart(int position, ImageView imageView, ProgressBar progressBar, View view)
	{
		//Check what kind of resource
		String type = mInterpretationList.get(position).mType;
		if(type.equals("chart"))
		{
			loadImageAsync(position, imageView, progressBar);
		}
		else if(type.equals("map"))
		{
			loadImageAsync(position, imageView, progressBar);
		}
		else if(type.equals("reportTable"))
		{
			RelativeLayout rlayout = (RelativeLayout) view.findViewById(R.id.interpretations_list_resource_container);
			
			TextView text = new TextView(mContext);
			text.setText("Unable to display resource of this type.");
			rlayout.removeView(imageView);
			rlayout.removeView(progressBar);
			rlayout.addView(text);
		}
		else if(type.equals("dataSetReport"))
		{
			RelativeLayout rlayout = (RelativeLayout) view.findViewById(R.id.interpretations_list_resource_container);
			
			TextView text = new TextView(mContext);
			text.setText("Unable to display resource of this type.");
			rlayout.removeView(imageView);
			rlayout.removeView(progressBar);
			rlayout.addView(text);
		}
		//If there is a chart to be drawn from the resource
		else
		{
			RelativeLayout rlayout = (RelativeLayout) view.findViewById(R.id.interpretations_list_resource_container);
			
			TextView text = new TextView(mContext);
			text.setText("Unable to display resource of this type.");
			rlayout.removeView(imageView);
			rlayout.removeView(progressBar);
			rlayout.addView(text);
		}
	}
	
	public void loadImageAsync(int position, ImageView imageView, ProgressBar progressBar)
	{
		CustomAsyncTask task = new CustomAsyncTask();
		Integer position_integer = position;
		Object[] obj = new Object[3];
		obj[0] = position_integer;
		obj[1] = imageView;
		obj[2] = progressBar;
		task.execute(obj);
	}
	
	/**
	 *	Used to load and display images asynchronously 
	 * @author Simen S. Russnes
	 *
	 */
	private class CustomAsyncTask extends AsyncTask<Object, Object, Drawable>
	{

		private Integer mmPosition;
		private ImageView mmImageView;
		private ProgressBar mmProgressBar;
		
		@Override
	    protected void onProgressUpdate(final Object... args) {
	        super.onProgressUpdate(args);
	    }
		
		@Override
		protected void onPreExecute() {
		}

		@Override 
		protected void onPostExecute(Drawable drawable) {
			if(drawable!=null)
			{
				mmImageView.setImageDrawable(drawable);
				mmImageView.setVisibility(View.VISIBLE);
				mmProgressBar.setVisibility(View.INVISIBLE);
			}
		}

		@Override
		protected Drawable doInBackground(Object... params) {
			// TODO Auto-generated method stub
			
			Object[] obj = params;
			System.out.println(obj.length);
			mmPosition = (Integer) params[0];
			mmImageView = (ImageView) params[1];
			mmProgressBar = (ProgressBar) params[2];
			Drawable drawable = ConnectionManager.getConnectionManager().getImage(mInterpretationList.get(mmPosition).getInterpretationDataUrl());
			return drawable;
		}
		
	}

}
