package no.uio.inf5750.assignment3.messaging;

import no.uio.inf5750.assignment3.R;
import no.uio.inf5750.assignment3.util.BaseActivity;
import no.uio.inf5750.assignment3.util.ConnectionManager;
import no.uio.inf5750.assignment3.util.UpdateDaemon;
import no.uio.inf5750.assignment3.util.Util;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.AdapterView.OnItemClickListener;

public class MessagingActivity extends BaseActivity {
	
	private ListView mListView;
	private Context mContext;
	String mMessages = "";
	private Button mButtonMessage;
	String[] mMessageIDs;
	
	/** 
	 * Called when the activity is first created. 
	 */
	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.messaging);
		mContext = this;
		setupViews();
		update();
	}
	
	@Override
	public void onResume(){
		super.onResume();
		
		update();
	}
	
	@Override
	public void showError(String error)
	{
		super.showError(error);
		
	}

	/**
	 * Initializes message listview and it's listener
	 */
	public void setupViews()
	{
		mButtonMessage = (Button) findViewById(R.id.messaging_newmessage);
		mButtonMessage.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				Intent intent = new Intent(mContext, WriteMessageActivity.class);
				startActivity(intent);
			}
		});
		
		mListView = (ListView) findViewById(R.id.messaging_listview);
		mListView.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view,
					int position, long id) {
					Intent intent = new Intent(mContext, MessageActivity.class);
					intent.putExtra("message", mMessageIDs[position]);
					intent.putExtra("messageIndex", position);
					startActivity(intent);
				}
		});

		Util.setSlidingDrawer(this);
	}
	
	public void switchAccount(int position)
	{
		System.out.println("switchAccounts");
		ConnectionManager.getConnectionManager().setAccount(position, mContext);
		UpdateDaemon.getDaemon().setCurrentDashboard(0);
		update();
	}

	/**
	 * Updates the list of messages
	 */
	public boolean update() {
		
		int verified = UpdateDaemon.getDaemon().verify();
		if(verified<11)
		{
			String error = "";
			if(verified==0) //00
			{
				error = "There is a problem with your account and your connection to the internet, please make sure your settings are correct";
				//connection error and account error
				System.out.println("double rror");
			}
			else if (verified==1) // 01
			{
				error = "There is a problem with your account. Please make sure your settings are correct";
				//account error but connection good
				System.out.println("account error");
			}
			else // 10
			{
				error = "There is a problem with your connection to the internet. Please make sure your connection settings are correct.";
				//connection error
				System.out.println("connection error");
			}
			
			showError(error);
			return false;
		}
		
		UpdateDaemon.getDaemon().updateMessagesAndInterpretations();
		int numMessages = UpdateDaemon.getDaemon().getNumberOfMessages();
		String[] values = new String[numMessages];
		String[] names = new String[numMessages];
		mMessageIDs = new String[numMessages];

		for (int i = 0; i < numMessages; i++) {
			values[i] = UpdateDaemon.getDaemon().getMessage(i).mTitle;
			names[i] = UpdateDaemon.getDaemon().getMessage(i).mUser.mName;
			mMessageIDs[i] = UpdateDaemon.getDaemon().getMessage(i).mId;
		}

		setListView(values, names);
		
		return true;
	}

	/**
	 * Sets the messagevalues in a listview
	 * @param values messagevalues used for the listview
	 */
	public void setListView(String[] values, String[] names)
	{
		if(values.length==0)
		{
			//There are no messages
			values = new String[]{"You have no messages"};
			ArrayAdapter<String> arrayAdapter = new ArrayAdapter<String>(mContext, R.layout.list_item, values);
			mListView.setAdapter(arrayAdapter);
			mListView.setClickable(false);
			mListView.setSelector(android.R.color.transparent);
			mListView.setOnItemClickListener(new OnItemClickListener() {
				public void onItemClick(AdapterView<?> parent, View view,
						int position, long id) {
						//Do nothing
					}
			});
			
		}
		else
		{
			MessageListArrayAdapter arrayAdapter = new MessageListArrayAdapter(mContext, R.layout.messages_list_item, values, names);
			mListView.setAdapter(arrayAdapter);
		}
	}
}
