package no.uio.inf5750.assignment3.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.LinkedList;
import java.util.Scanner;

import org.apache.commons.codec.binary.Base64;


import dhis2.org.apache.http.HeaderElement;
import dhis2.org.apache.http.HttpEntity;
import dhis2.org.apache.http.HttpHost;
import dhis2.org.apache.http.HttpResponse;
import dhis2.org.apache.http.client.ClientProtocolException;
import dhis2.org.apache.http.client.HttpClient;
import dhis2.org.apache.http.client.cache.CacheResponseStatus;
import dhis2.org.apache.http.client.cache.HttpCacheContext;
import dhis2.org.apache.http.client.config.RequestConfig;
import dhis2.org.apache.http.client.methods.CloseableHttpResponse;
import dhis2.org.apache.http.client.methods.HttpGet;
import dhis2.org.apache.http.client.methods.HttpPost;
import dhis2.org.apache.http.entity.StringEntity;
import dhis2.org.apache.http.entity.mime.Header;
import dhis2.org.apache.http.impl.client.CloseableHttpClient;
import dhis2.org.apache.http.impl.client.DefaultHttpClient;
import dhis2.org.apache.http.impl.client.cache.CacheConfig;
import dhis2.org.apache.http.impl.client.cache.CachingHttpClients;

import android.net.ConnectivityManager;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.DrawableContainer;

/**
 * ConnectionManager is used to communicate with the server and store
 * authentication information
 * @author Jan Anders Bremer
 */

public class ConnectionManager {
	private static ConnectionManager mManager = null;
	private CacheConfig mCacheConfig = null; 
	private RequestConfig mRequestConfig = null;
	private CloseableHttpClient mCachingClient = null;
	private HttpCacheContext mContext = null;
	private long mCacheObjectSizeInBytes = 200000;
	
	private String mUsername, mPassword, mLog, mSite;

	/**
	 * Makes this class a singleton
	 */
	static {
		mManager =  new ConnectionManager();
	}

	/**
	 * Only a private constructor is available for this class
	 */
	private ConnectionManager() 
	{
		mLog = "";
		initiateCache();
	}
	
	/**
	 * Changes the site the app uses
	 * @param site The new site
	 */
	public void setSite(String site) {
		mSite = site;
	}
	
	/**
	 * @return The current site URL.
	 */
	public String getSite() {
		return mSite;
	}

	/**
	 * @return The ConnectionManager instance
	 */
	public static ConnectionManager getConnectionManager() 
	{
		return mManager;
	}
	
	/**
	 * @return Username of the current user
	 */
	public String getUsername() 
	{
		return mUsername;
	}

	/**
	 * Changes username
	 * @param username The new user
	 */
	public void setUsername(String username) 
	{
		mUsername = username;
	}

	/**
	 * Changes password
	 * @param password New password
	 */
	public void setPassword(String password) 
	{
		mPassword = password;
	}
	
	/**
	 * Used to reply to a message
	 * @param id Position in message-array
	 * @param message The reply
	 */
	public void replyMessage(int id, String message) {
		doPost(UpdateDaemon.getDaemon().getMessage(id).getConversationUrl(), message);
	}
	
	/**
	 * Used to reply to a message
	 * @param msgId String containing the messageId
	 * @param message The reply 
	 */
	public void replyMessage(String msgId, String message) {
		doPost(Message.getConversationUrl(msgId), message);
	}
	
	public void initiateCache()
	{

		mCacheConfig = CacheConfig.custom()
		        .setMaxCacheEntries(1000)
		        .setMaxObjectSize(mCacheObjectSizeInBytes)
		        .build();
		mRequestConfig = RequestConfig.custom()
		        .setConnectTimeout(0)
		        .setSocketTimeout(0)
		        .build();
		mCachingClient = CachingHttpClients.custom()
		        .setCacheConfig(mCacheConfig)
		        .setDefaultRequestConfig(mRequestConfig)
		        .build();
		
		mContext = HttpCacheContext.create();
		
	}
	
	/**
	 * Sends a message to the specified users
	 * @param users
	 * @param message
	 */
	public void sendMessage(LinkedList<User> users, String message, String subject)
	{
		String xml = "<message xmlns=\"http://dhis2.org/schema/dxf/2.0\"><subject>" + 
				subject + "</subject><text>" + message + "</text><users>";
		
		String usersXML = "";
		for(int i = 0; i<users.size(); i++)
		{
			usersXML += "<user id=\"" + users.get(i).mId+"\" />";
		}
		xml += usersXML + "</users>";
		xml += "</message>";
		System.out.println(xml);
		
		doPost(Message.getConversationBaseUrl(), xml, "application/xml");
	}
	
	/**
	 * Adds a comment to an interpretation
	 * @param interId String containg the interpretation Id
	 * @param message The reply
	 */
	public void replyInterpretation(String interId, String message) {
		doPost(Interpretation.getInterpretationUrl(interId), message);
	}

	/**
	 * @return Base64 encoded username and password.
	 */
	private String getEncodedUserInfo() {
		return new String(Base64.encodeBase64((mUsername+':'+mPassword).getBytes()));
	}
	
	/**
	 * Retrieves a given image
	 * @param url The location of the image
	 * @return Drawable containing the image
	 */
	public Drawable getImage(String url)
	{
		Drawable drawable = null;

		HttpGet httpget = new HttpGet(url);
		httpget.setHeader("Authorization", "Basic " + getEncodedUserInfo());

		CloseableHttpResponse response = null;
		
		try {
			response = mCachingClient.execute(httpget, mContext);
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		//cachingClient.getConnectionManager().shutdown();
		
		try {
		    CacheResponseStatus responseStatus = mContext.getCacheResponseStatus();
		    switch (responseStatus) {
		        case CACHE_HIT:
		            System.out.println("A response was generated from the cache with " +
		                    "no requests sent upstream");
		            break;
		        case CACHE_MODULE_RESPONSE:
		            System.out.println("The response was generated directly by the " +
		                    "caching module");
		            break;
		        case CACHE_MISS:
		            System.out.println("The response came from an upstream server");
		            break;
		        case VALIDATED:
		            System.out.println("The response was generated from the cache " +
		                    "after validating the entry with the origin server");
		            break;
		    }
		}catch(Exception e)
		{
			
		}

		HttpEntity entity = null;
		if(response!=null)
		{
			entity = response.getEntity();
		}

		InputStream is = null;
		if(entity!=null)
		{
			try {
				is = entity.getContent();
				drawable = Drawable.createFromStream(is, null);
				
			} catch (IllegalStateException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (OutOfMemoryError e)
			{
				e.printStackTrace();
			}
		}
		else
		{
			System.out.println("entity is null!");
		}
		
		
		try {
			if(is!=null)
				is.reset();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		try {
			response.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return drawable;
	}
	
	/**
	 * Executes a HTTP post with a given content
	 * @param url POST URL
	 * @param content The content to be sent in the post
	 */
	public void doPost(String url, String content) {
		HttpPost httppost = new HttpPost(url);
		httppost.setHeader("Authorization", "Basic " + getEncodedUserInfo());
		try {
			httppost.setEntity(new StringEntity(content));
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return;
		}
		//HttpClient httpclient = new DefaultHttpClient();
		
		CloseableHttpResponse response = null;
		try {
			response = mCachingClient.execute(httppost, mContext);
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		try {
			response.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		//System.out.println("url: " + url);
		
	}
	
	/**
	 * Executes a HTTP post with a given content
	 * @param url POST URL
	 * @param contentType the content type of the request
	 * @param content The content to be sent in the post
	 */
	public void doPost(String url, String content, String contentType) {
		HttpPost httppost = new HttpPost(url);
		httppost.setHeader("Content-Type", contentType);
		httppost.setHeader("Authorization", "Basic " + getEncodedUserInfo());
		
		try {
			httppost.setEntity(new StringEntity(content));
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return;
		}
		//HttpClient httpclient = new DefaultHttpClient();
		
		
		CloseableHttpResponse response = null;
		try {
			response = mCachingClient.execute(httppost, mContext);
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		try {
			response.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		//System.out.println("url: " + url);
		
	}
	
	public void setAccount(int position, Context context)
	{
		
		LinkedList<String[]> accounts = Util.getAccounts(context);
		String server = accounts.get(position)[1];
		String username = accounts.get(position)[2];
		String password = accounts.get(position)[3];
		ConnectionManager.getConnectionManager().setUsername(username);
		ConnectionManager.getConnectionManager().setPassword(password);
		ConnectionManager.getConnectionManager().setSite(server);
		
	}
	
	/**
	 * Retrieves a given url 
	 * @param url The url retrieved
	 * @return The content of what the url is pointing at
	 */
	public String doRequest(String url)
	{
		String output = "";

		//HttpClient httpclient = new DefaultHttpClient();

		HttpGet httpget = new HttpGet(url);
		httpget.setHeader("Authorization", "Basic " + getEncodedUserInfo());

		CloseableHttpResponse response = null;
		try {
			response = mCachingClient.execute(httpget, mContext);
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
		HttpEntity entity = null;
		if(response!=null)
		{
			entity = response.getEntity();
		}

		if(entity!=null)
		{
			try {
				InputStream is = entity.getContent();
				Scanner sc = new Scanner(is);
				output = "";
				while (sc.hasNext()) {
					output += sc.nextLine();
				}

			} catch (IllegalStateException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		try {
			response.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return output;

	}
	
	public String doRequestNoCache(String url)
	{
		String output = "";
		
		HttpClient httpclient = new DefaultHttpClient();

		HttpGet httpget = new HttpGet(url);
		httpget.setHeader("Authorization", "Basic " + getEncodedUserInfo());

		HttpResponse response = null;
		try {
			response = httpclient.execute(httpget, mContext);
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
		HttpEntity entity = null;
		if(response!=null)
		{
			entity = response.getEntity();
		}

		if(entity!=null)
		{
			try {
				InputStream is = entity.getContent();
				Scanner sc = new Scanner(is);
				output = "";
				while (sc.hasNext()) {
					output += sc.nextLine();
				}

			} catch (IllegalStateException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return output;
	}
	
	public int verify()
	{
		int code = -1;

		HttpGet httpget = new HttpGet(ConnectionManager.getConnectionManager().getSite() + "/api");
		httpget.setHeader("Authorization", "Basic " + getEncodedUserInfo());

		CloseableHttpResponse response = null;
		try {
			response = mCachingClient.execute(httpget, mContext);
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return code;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return code;
		} catch (Exception e)
		{
			e.printStackTrace();
			return code;
		}
		
		if(response!=null)
		{
			dhis2.org.apache.http.Header content_type = response.getEntity().getContentType();
			if(content_type.getValue().contains("json"))
			{//I think this means it is verified
				code = 0;
			}
			
			try {
				response.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		return code;
	}

}
