package no.uio.inf5750.assignment3.util;

import java.io.IOException;
import java.io.StringReader;
import java.util.LinkedList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import no.uio.inf5750.assignment3.R;
import no.uio.inf5750.assignment3.accounts.Accounts;
import no.uio.inf5750.assignment3.dashboard.DashboardActivityNew;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.SlidingDrawer;
import android.widget.AdapterView.OnItemClickListener;

public class Util {
	
	private final static String PREFS_NAME = "PREFERENCES";
	
	public static void displayErrorDialog(final Activity activity, String errorMessage)
	{
		AlertDialog.Builder alertDialogBuilder = new AlertDialog.Builder(
				activity);

		// set title
		alertDialogBuilder.setTitle("A problem has occured!");

		// set dialog message
		alertDialogBuilder.setMessage(errorMessage);
		alertDialogBuilder.setCancelable(false);
		alertDialogBuilder.setPositiveButton("Accounts",
				new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int id) {
						// if this button is clicked, close
						// current activity
						dialog.cancel();
						Intent intent = new Intent(activity, Accounts.class);
						activity.startActivity(intent);
						
					}
				});
		alertDialogBuilder.setNegativeButton("Close",
				new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int id) {
						// if this button is clicked, just close
						// the dialog box and do nothing
						dialog.cancel();
						//activity.finish();
					}
				});

		// create alert dialog
		AlertDialog alertDialog = alertDialogBuilder.create();

		// show it
		alertDialog.show();
	}
	
	public static void updateSlidingDrawer(final BaseActivity activity)
	{
		ListView listViewSlidingDrawerAccounts = (ListView) activity.findViewById(R.id.slidingdrawer_accountslist);
		
		String[] accounts = Util.getAccountNames(activity);
		ArrayAdapter<String> adapter2 = new ArrayAdapter<String>(activity, R.layout.modules_list_item, accounts);
		listViewSlidingDrawerAccounts.setAdapter(adapter2);
	}
	
	public static void setSlidingDrawer(final BaseActivity activity)
	{
		RelativeLayout root = (RelativeLayout) activity.findViewById(R.id.root_layout);
		final SlidingDrawer slidingDrawer = (SlidingDrawer) activity.getLayoutInflater().inflate(R.layout.slidingdrawer, root, false);
		if(root==null)
			System.out.println("root == null");
		else if(slidingDrawer==null)
			System.out.println("sldigndrawer==null");
		root.addView(slidingDrawer);
		ListView listViewSlidingDrawer = (ListView) activity.findViewById(R.id.slidingdrawer_moduleslist);
		String[] modules = new String[]{"Dashboard", "Messages", "Interpretations", "Edit Accounts"};
		ModulesListArrayAdapter adapter = new ModulesListArrayAdapter(activity, R.layout.modules_list_item, modules);
		listViewSlidingDrawer.setAdapter(adapter);
		listViewSlidingDrawer.setOnItemClickListener(new OnItemClickListener() {
            public void onItemClick(AdapterView<?> parent, View view,
                int position, long id) {
            	
            	if(position == Globals.getGlobals().getCurrentModule())
            	{//Don't relaunch current activity
            		slidingDrawer.close();
            		return;
            	}
            	
            	slidingDrawer.toggle();
            	activity.launchActivity(position);    
            }
          });
		
		ListView listViewSlidingDrawerAccounts = (ListView) activity.findViewById(R.id.slidingdrawer_accountslist);
		
		String[] accounts = Util.getAccountNames(activity);
		ArrayAdapter<String> adapter2 = new ArrayAdapter<String>(activity, R.layout.modules_list_item, accounts);
		listViewSlidingDrawerAccounts.setAdapter(adapter2);
		listViewSlidingDrawerAccounts.setOnItemClickListener(new OnItemClickListener() {
            public void onItemClick(AdapterView<?> parent, View view,
                int position, long id) {
            	
            	slidingDrawer.toggle();
            	activity.switchAccount(position);
            	
            }
          });
		Button drawerHandle = (Button) activity.findViewById(R.id.button_drawerhandle);
		drawerHandle.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				slidingDrawer.animateToggle();
			}
		});
	}
	
	
	
	

	public static Document getDomElement(String xml)
	{
		Document doc = null;
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		try {

			DocumentBuilder db = dbf.newDocumentBuilder();

			InputSource is = new InputSource(new StringReader(xml));
			doc = db.parse(is);

		} catch (ParserConfigurationException e) {
			Log.e("Error: ", e.getMessage());
			e.printStackTrace();
			return null;
		} catch (SAXException e) {
			Log.e("Error: ", e.getMessage());
			e.printStackTrace();
			return null;
		} catch (IOException e) {
			Log.e("Error: ", e.getMessage());
			e.printStackTrace();
			return null;
		}
		// return DOM
		return doc;
	}

	public static String getValue(Element item, String str) {
		NodeList n = item.getElementsByTagName(str);
		return getElementValue(n.item(0));
	}

	public static final String getElementValue( Node elem ) 
	{
		Node child;
		if( elem != null){
			if (elem.hasChildNodes()){
				for( child = elem.getFirstChild(); child != null; child = child.getNextSibling() ){
					if( child.getNodeType() == Node.TEXT_NODE  ){
						return child.getNodeValue();
					}
				}
			}
		}
		return "";
	}
	
	public static void storeAccount(Context context, String[] account)
	{
		SharedPreferences settings = context.getSharedPreferences(PREFS_NAME, 0);
        SharedPreferences.Editor editor = settings.edit();
        int accountsCount = getAccountsCount(context);
    	
        account[0] = account[0].trim();
        account[1] = account[1].trim();
        account[2] = account[2].trim();
        account[3] = account[3].trim();
        
        //Make sure all fields are filled in
        if(account[0].length()==0 || account[1].length()==0 || account[2].length()==0 || account[3].length()==0)
        	return;
        
        //Check that server address doesn't have / at the end. We add that later so no need
        if(account[1].charAt(account[1].length()-1) == '/')
        {
        	account[1] = account[1].substring(0, account[1].length()-1);
        }
        
        for(int i = 0; i<account.length; i++)
        {
        	System.out.println("."+account[i]+".");
        }
        
        editor.putString("readableName"+accountsCount, account[0]);
        editor.putString("server"+accountsCount, account[1]);
        editor.putString("username"+accountsCount, account[2]);
        editor.putString("password"+accountsCount, account[3]);
        accountsCount ++;
        editor.putInt("accountsCount", accountsCount);
        editor.commit();
	}
	
	public static LinkedList<String[]> getAccounts(Context context)
	{
		SharedPreferences settings = context.getSharedPreferences(PREFS_NAME, 0);
		int accountsCount = getAccountsCount(context);
		LinkedList<String[]> accounts = new LinkedList<String[]>();
		
		for(int i = 0; i<accountsCount; i++)
        {
        	String readableName = null;
        	String username = null;
        	String password = null;
        	String server = null;
        	readableName = settings.getString("readableName"+i, "null");
        	username = settings.getString("username"+i, "null");
        	password = settings.getString("password"+i, "null");
        	server = settings.getString("server"+i, "null");
        	
        	String[] account = new String[]{readableName, server, username, password};
        	accounts.add(account);
        }
		return accounts;
	}
	
	public static String[] getAccountNames(Context context)
	{
		LinkedList<String[]> accounts = getAccounts(context);
		String[] accountNames = new String[accounts.size()];
    	for(int i = 0; i<accounts.size(); i++)
    	{
    		accountNames[i] = accounts.get(i)[0];
    	}
    	
    	return accountNames;
	}
	
	public static int getAccountsCount(Context context)
	{
		SharedPreferences settings = context.getSharedPreferences(PREFS_NAME, 0);
		int accountsCount = 0;
        accountsCount = settings.getInt("accountsCount", 0);
        return accountsCount;
	}
	
	public static void removeAccount(Context context, int position)
	{
		LinkedList<String[]> accounts = getAccounts(context);
		accounts.remove(position);
		storeAccounts(context, accounts);
		
	}
	
	public static void storeAccounts(Context context, LinkedList<String[]> accounts){
		
		SharedPreferences settings = context.getSharedPreferences(PREFS_NAME, 0);
        SharedPreferences.Editor editor = settings.edit();
        //int accountsCount = getAccountsCount(context);
        
        for(int i = 0; i<accounts.size(); i++)
        {
        	String[] account = accounts.get(i);
        	editor.putString("readableName"+i, account[0]);
            editor.putString("server"+i, account[1]);
            editor.putString("username"+i, account[2]);
            editor.putString("password"+i, account[3]);
        }
        editor.putInt("accountsCount", accounts.size());
        editor.commit();
		
	}
	
	public static void setAccountsCount(Context context, int accountsCount)
	{
		SharedPreferences settings = context.getSharedPreferences(PREFS_NAME, 0);
        SharedPreferences.Editor editor = settings.edit();
		editor.putInt("accountsCount", accountsCount);
	}

}
